#!/usr/bin/env python
# -*- coding: utf-8 -*-


def is_string_type(needle):
    if isinstance(needle, str):
        return True
    else:
        return False


def is_int_type(needle):
    if isinstance(needle, int):
        return True
    else:
        return False


def is_int_in_a_string(needle):
    try:
        int(needle)
        return True
    except ValueError:
        return False


def is_float_type(needle):
    if isinstance(needle, float):
        return True
    else:
        return False


def is_float_in_a_string(needle):
    try:
        float(needle)
        return True
    except ValueError:
        return False


def is_int_or_float_type(needle):
    if is_int_type(needle) is True or is_float_type(needle) is True:
        return True
    else:
        return False


def is_list_type(needle):
    if isinstance(needle, list):
        return True
    else:
        return False



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os.path
import xml.etree.ElementTree as ETree

from ufolint.data.tstobj import Result
from ufolint.data.ufo import Ufo2, Ufo3
from ufolint.stdoutput import StdStreamer
from ufolint.utilities import file_exists

from fontTools.ufoLib import UFOReader
from fontTools.ufoLib.glifLib import GlyphSet


class AbstractPlistValidator(object):
    def __init__(self, ufopath, ufoversion, glyphs_dir_list):
        self.ufopath = ufopath
        self.ufoversion = ufoversion
        self.glyphs_dir_list = glyphs_dir_list
        self.testfile = None
        if self.ufoversion == 2:
            self.ufoobj = Ufo2(self.ufopath, self.glyphs_dir_list)
        elif self.ufoversion == 3:
            self.ufoobj = Ufo3(self.ufopath, self.glyphs_dir_list)
        self.root_plist_list = self.ufoobj.all_root_plist_files_list
        self.glyphsdir_plist_list = self.ufoobj.all_glyphsdir_plist_files_list
        self.mandatory_filepaths_list = self.ufoobj.get_mandatory_filepaths_list()
        self.test_fail_list = []

    def _parse_xml(self, testpath):
        res = Result(testpath)
        try:
            ETree.parse(testpath)
            res.test_failed = False
            return res
        except Exception as e:
            res.test_failed = True
            if (
                testpath in self.mandatory_filepaths_list
            ):  # if this test is on a mandatory file, exit early with fail
                res.exit_failure = True
            res.test_long_stdstream_string = (
                testpath + " failed XML validation test with error: " + str(e)
            )
            self.test_fail_list.append(res)  # add each failure to test failures list
            return res

    def run_xml_validation(self):
        ss = StdStreamer(self.ufopath)
        if self.testfile in self.root_plist_list:
            testpath = self.ufoobj.get_root_plist_filepath(self.testfile)
            if file_exists(testpath):
                res = self._parse_xml(testpath)
                ss.stream_result(res)
            else:  # there is no file to check, mandatory files have already been checked, this is a success
                res = Result(testpath)
                res.test_failed = False
                ss.stream_result(res)
        elif self.testfile in self.glyphsdir_plist_list:
            testpath_list = self.ufoobj.get_glyphsdir_plist_filepath_list(self.testfile)
            for testpath in testpath_list:
                if file_exists(testpath):
                    res = self._parse_xml(testpath)
                    ss.stream_result(res)
                else:  # there is no file to check, mandatory files have already been checked, this is a success
                    res = Result(testpath)
                    res.test_failed = False
                    ss.stream_result(res)
        return (
            self.test_fail_list
        )  # return to the calling code so that it can be maintained for final user report

    def run_ufolib_import_validation(self):
        raise NotImplementedError


class MetainfoPlistValidator(AbstractPlistValidator):
    def __init__(self, ufopath, ufoversion, glyphs_dir_list):
        super(MetainfoPlistValidator, self).__init__(
            ufopath, ufoversion, glyphs_dir_list
        )
        self.testfile = "metainfo.plist"
        self.testpath = self.ufoobj.get_root_plist_filepath(self.testfile)

    def run_ufolib_import_validation(self):
        """
        ufoLib UFOReader.readMetaInfo method validates the UFO version number.  This method adds validation for
        expected reverse URL name scheme in
        :return: (list) list of test failure Result objects
        """

        res = Result(self.testpath)
        ss = StdStreamer(self.ufopath)
        if (
            file_exists(self.testpath) is False
        ):  # fail test, exit early if file is missing
            res.test_failed = True
            res.exit_failure = True
            res.test_long_stdstream_string = (
                "metainfo.plist is not available on the path " + self.testpath
            )
            ss.stream_result(res)
        try:
            ufolib_reader = UFOReader(self.ufopath, validate=True)
            ufolib_reader.readMetaInfo()
            res.test_failed = False
            ss.stream_result(res)
        except Exception as e:
            res.test_failed = True
            res.exit_failure = True
            res.test_long_stdstream_string = (
                self.testpath + " failed ufoLib import test with error: " + str(e)
            )
            self.test_fail_list.append(res)
            ss.stream_result(res)
        return self.test_fail_list


class FontinfoPlistValidator(AbstractPlistValidator):
    def __init__(self, ufopath, ufoversion, glyphs_dir_list):
        super(FontinfoPlistValidator, self).__init__(
            ufopath, ufoversion, glyphs_dir_list
        )
        self.testfile = "fontinfo.plist"
        self.testpath = self.ufoobj.get_root_plist_filepath(self.testfile)

        class FontInfoObj(object):
            def __init__(self):
                pass

        self.fontinfo_obj = FontInfoObj()

    def run_ufolib_import_validation(self):
        """
        ufoLib UFOReader.readInfo method validates value types in the fontinfo.plist file
        :return: (list) list of test failure Result objects
        """
        res = Result(self.testpath)
        ss = StdStreamer(self.ufopath)
        if file_exists(self.testpath) is False:
            res.test_failed = (
                False
            )  # not a mandatory file in UFO spec, test passes if missing
            ss.stream_result(res)
            return self.test_fail_list
        try:
            # read fontinfo.plist with ufoLib - the ufoLib library performs type validations on values on read
            ufolib_reader = UFOReader(self.ufopath, validate=True)
            ufolib_reader.readInfo(self.fontinfo_obj)
            res.test_failed = False
            ss.stream_result(res)
        except Exception as e:
            res.test_failed = True
            res.test_long_stdstream_string = (
                self.testpath + " failed ufoLib import test with error: " + str(e)
            )
            ss.stream_result(res)
            self.test_fail_list.append(res)
        return self.test_fail_list


class GroupsPlistValidator(AbstractPlistValidator):
    def __init__(self, ufopath, ufoversion, glyphs_dir_list):
        super(GroupsPlistValidator, self).__init__(ufopath, ufoversion, glyphs_dir_list)
        self.testfile = "groups.plist"
        self.testpath = self.ufoobj.get_root_plist_filepath(self.testfile)

    def run_ufolib_import_validation(self):
        """
        ufoLib UFOReader.readGroups method validates the groups.plist file
        :return: (list) list of test failure Result objects
        """
        res = Result(self.testpath)
        ss = StdStreamer(self.ufopath)
        if file_exists(self.testpath) is False:
            res.test_failed = (
                False
            )  # not a mandatory file in UFO spec, test passes if missing
            ss.stream_result(res)
            return self.test_fail_list
        try:
            # read groups.plist with ufoLib - the ufoLib library performs type validations on values on read
            ufolib_reader = UFOReader(self.ufopath, validate=True)
            ufolib_reader.readGroups()
            res.test_failed = False
            ss.stream_result(res)
        except Exception as e:
            res.test_failed = True
            res.test_long_stdstream_string = (
                self.testpath + " failed ufoLib import test with error: " + str(e)
            )
            ss.stream_result(res)
            self.test_fail_list.append(res)
        return self.test_fail_list


class KerningPlistValidator(AbstractPlistValidator):
    def __init__(self, ufopath, ufoversion, glyphs_dir_list):
        super(KerningPlistValidator, self).__init__(
            ufopath, ufoversion, glyphs_dir_list
        )
        self.testfile = "kerning.plist"
        self.testpath = self.ufoobj.get_root_plist_filepath(self.testfile)

    def run_ufolib_import_validation(self):
        """
        ufoLib UFOReader.readKerning method validates the kerning.plist file
        :return: (list) list of test failure Result objects
        """
        res = Result(self.testpath)
        ss = StdStreamer(self.ufopath)
        if file_exists(self.testpath) is False:
            res.test_failed = (
                False
            )  # not a mandatory file in UFO spec, test passes if missing
            ss.stream_result(res)
            return self.test_fail_list
        try:
            # read kerning.plist with ufoLib - the ufoLib library performs type validations on values on read
            ufolib_reader = UFOReader(self.ufopath, validate=True)
            ufolib_reader.readKerning()
            res.test_failed = False
            ss.stream_result(res)
        except Exception as e:
            res.test_failed = True
            res.test_long_stdstream_string = (
                self.testpath + " failed ufoLib import test with error: " + str(e)
            )
            ss.stream_result(res)
            self.test_fail_list.append(res)
        return self.test_fail_list


class LibPlistValidator(AbstractPlistValidator):
    def __init__(self, ufopath, ufoversion, glyphs_dir_list):
        super(LibPlistValidator, self).__init__(ufopath, ufoversion, glyphs_dir_list)
        self.testfile = "lib.plist"
        self.testpath = self.ufoobj.get_root_plist_filepath(self.testfile)

    def run_ufolib_import_validation(self):
        """
        ufoLib UFOReader.readLib method validates the lib.plist file
        :return: (list) list of test failure Result objects
        """
        res = Result(self.testpath)
        ss = StdStreamer(self.ufopath)
        if file_exists(self.testpath) is False:
            res.test_failed = (
                False
            )  # not a mandatory file in UFO spec, test passes if missing
            ss.stream_result(res)
            return self.test_fail_list
        try:
            # read lib.plist with ufoLib - the ufoLib library performs type validations on values on read
            ufolib_reader = UFOReader(self.ufopath, validate=True)
            ufolib_reader.readLib()
            res.test_failed = False
            ss.stream_result(res)
        except Exception as e:
            res.test_failed = True
            res.test_long_stdstream_string = (
                self.testpath + " failed ufoLib import test with error: " + str(e)
            )
            ss.stream_result(res)
            self.test_fail_list.append(res)
        return self.test_fail_list


class ContentsPlistValidator(AbstractPlistValidator):
    def __init__(self, ufopath, ufoversion, glyphs_dir_list):
        super(ContentsPlistValidator, self).__init__(
            ufopath, ufoversion, glyphs_dir_list
        )
        self.testfile = (
            "contents.plist"
        )  # can occur in multiple glyphs directories in UFOv3+
        self.glyphs_dir_list = glyphs_dir_list

    def run_ufolib_import_validation(self):
        """
        ufoLib GlyphSet instantiation validates the contents.plist file
        :return: (list) list of test failure Result objects
        """
        ss = StdStreamer(self.ufopath)
        for glyphs_dir in self.ufoobj.glyphsdir_list:
            res = Result(glyphs_dir[1])
            rel_dir_path = os.path.join(self.ufopath, glyphs_dir[1])
            try:
                # read contents.plist with ufoLib as GlyphSet instantiation
                # the ufoLib library performs type validations on values on read
                # glyphs_dir_list is a list of lists mapped to glyphs dir name, glyphs dir path
                GlyphSet(
                    rel_dir_path, ufoFormatVersion=self.ufoversion, validateRead=True
                )  # test for raised exceptions
                res.test_failed = False
                ss.stream_result(res)
            except Exception as e:
                res.test_failed = True
                res.exit_failure = True  # mandatory file
                res.test_long_stdstream_string = (
                    "contents.plist in "
                    + rel_dir_path
                    + " failed ufoLib import test with error: "
                    + str(e)
                )
                self.test_fail_list.append(res)
                ss.stream_result(res)
        return self.test_fail_list


class LayercontentsPlistValidator(AbstractPlistValidator):
    def __init__(self, ufopath, ufoversion, glyphs_dir_list):
        super(LayercontentsPlistValidator, self).__init__(
            ufopath, ufoversion, glyphs_dir_list
        )
        self.testfile = "layercontents.plist"
        self.testpath = self.ufoobj.get_root_plist_filepath(self.testfile)

    def run_ufolib_import_validation(self):
        """
        ufoLib UFOReader.getLayerNames method validates the layercontents.plist file
        :return: (list) list of test failure Result objects
        """
        res = Result(self.testpath)
        ss = StdStreamer(self.ufopath)
        if (
            file_exists(self.testpath) is False
        ):  # should only meet this condition if not a mandatory file (runner.py checks)
            res.test_failed = False
            ss.stream_result(res)
            return self.test_fail_list
        try:
            # read layercontents.plist with ufoLib - the ufoLib library performs type validations on values on read
            ufolib_reader = UFOReader(self.ufopath, validate=True)
            ufolib_reader.getLayerNames()
            res.test_failed = False
            ss.stream_result(res)
        except Exception as e:
            if (
                self.testpath in self.mandatory_filepaths_list
            ):  # if part of mandatory file spec for UFO version, fail early
                res.test_failed = True
                res.exit_failure = True  # fail early b/c it is mandatory part of spec
            else:
                res.test_failed = (
                    True
                )  # fail the test, but wait to report until all other tests complete
            res.test_long_stdstream_string = (
                self.testpath + " failed ufoLib import test with error: " + str(e)
            )
            self.test_fail_list.append(res)
            ss.stream_result(res)
        return self.test_fail_list


class LayerinfoPlistValidator(AbstractPlistValidator):
    def __init__(self, ufopath, ufoversion, glyphs_dir_list):
        super(LayerinfoPlistValidator, self).__init__(
            ufopath, ufoversion, glyphs_dir_list
        )
        self.testfile = "layerinfo.plist"

        class LayerInfoObj(object):
            def __init__(self):
                pass

        self.layerinfo_obj = LayerInfoObj()

    def run_ufolib_import_validation(self):
        """
        ufoLib GlyphSet.readLayerInfo method performs validations of layerinfo.plist file(s)
        :return: (list) list of test failure Result objects
        """
        ss = StdStreamer(self.ufopath)
        for glyphs_dir in self.ufoobj.glyphsdir_list:
            res = Result(glyphs_dir[1])
            rel_dir_path = os.path.join(self.ufopath, glyphs_dir[1])

            try:
                gs = GlyphSet(
                    rel_dir_path, ufoFormatVersion=self.ufoversion, validateRead=True
                )
                gs.readLayerInfo(self.layerinfo_obj)
                res.test_failed = False
                ss.stream_result(res)
            except Exception as e:
                res.test_failed = True
                res.test_long_stdstream_string = (
                    "layerinfo.plist in "
                    + rel_dir_path
                    + " failed ufoLib import test with error: "
                    + str(e)
                )
                self.test_fail_list.append(res)
                ss.stream_result(res)
        return self.test_fail_list



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import xml.etree.ElementTree as Etree  # python 3


def is_valid_xml_path(filepath):
    """
    Attempts to parse XML file at filepath.  ElementTree.parse() raises informative exceptions as ParseError if cannot
    parse as valid XML.  This function catches them and returns as part of the function response to caller.

    Returns tuple of (boolean, message string) where True = valid XML, False = invalid XML.

    :param filepath: (string) filepath to attempt to parse as XML formatted text file
    :return: (boolean, message string) tuple
    """
    try:
        tree = Etree.parse(filepath)
        return True, tree
    except Exception as e:
        return False, str(e)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os.path


class Ufo(object):
    def __init__(self, ufopath, glyphsdir_list):
        self.ufopath = ufopath
        self.ufoversion = (
            None
        )  # defined on instantiation in the classes that inherit Ufo
        self.glyphsdir_list = glyphsdir_list
        self.mandatory_root_basefilepaths = None
        self.mandatory_glyphsdir_basefilepaths = None
        self.all_root_plist_files_list = None
        self.all_glyphsdir_plist_files_list = None

    def _make_root_plist_path(self, basefilename):
        if basefilename == "metainfo.plist":
            return os.path.join(self.ufopath, basefilename)
        elif basefilename == "fontinfo.plist":
            return os.path.join(self.ufopath, basefilename)
        elif basefilename == "groups.plist":
            return os.path.join(self.ufopath, basefilename)
        elif basefilename == "kerning.plist":
            return os.path.join(self.ufopath, basefilename)
        elif basefilename == "lib.plist":
            return os.path.join(self.ufopath, basefilename)
        elif basefilename == "layercontents.plist":
            return os.path.join(self.ufopath, basefilename)

    def _make_glyphsdir_plist_path(self, glyphsdirname, basefilename):
        if basefilename == "contents.plist":
            return os.path.join(self.ufopath, glyphsdirname, basefilename)
        elif basefilename == "layerinfo.plist":
            return os.path.join(self.ufopath, glyphsdirname, basefilename)

    def get_root_plist_filepath(self, basefilename):
        return self._make_root_plist_path(basefilename)

    def get_glyphsdir_plist_filepath_list(self, basefilename):
        path_list = []
        for glyphsdir in self.glyphsdir_list:
            glyphsdir_basename = glyphsdir[1]
            path_list.append(
                self._make_glyphsdir_plist_path(glyphsdir_basename, basefilename)
            )
        return path_list

    def get_mandatory_filepaths_list(self):
        """
        Creates a list of relative filepaths to mandatory files in UFO source.  These files are defined in the
        Ufo2 and Ufo3 classes that inherit from Ufo
        :return: list of filepath strings
        """
        mandatory_filepath_list = []
        for mandatory_root_basefile in self.mandatory_root_basefilepaths:
            mandatory_filepath_list.append(
                self.get_root_plist_filepath(mandatory_root_basefile)
            )
        for mandatory_glyphs_basefile in self.mandatory_glyphsdir_basefilepaths:
            glyphsdirs_filelist = self.get_glyphsdir_plist_filepath_list(
                mandatory_glyphs_basefile
            )
            for a_file in glyphsdirs_filelist:
                mandatory_filepath_list.append(a_file)
        return mandatory_filepath_list

    def get_glyphsdir_path_list(self):
        raise NotImplementedError

    def get_ufo_version(self):
        return self.ufoversion


class Ufo2(Ufo):
    def __init__(self, ufopath, glyphsdir_list):
        super(Ufo2, self).__init__(ufopath, glyphsdir_list)
        self.ufopath = ufopath
        self.ufoversion = 2
        self.glyphsdir_list = glyphsdir_list
        self.mandatory_root_basefilepaths = ["metainfo.plist"]
        self.mandatory_glyphsdir_basefilepaths = ["contents.plist"]
        self.all_root_plist_files_list = [
            "metainfo.plist",
            "fontinfo.plist",
            "groups.plist",
            "kerning.plist",
            "lib.plist",
        ]
        self.all_glyphsdir_plist_files_list = ["contents.plist"]

    def get_glyphsdir_path_list(self):
        glyphsdir_path = os.path.join(
            self.ufopath, "glyphs"
        )  # UFOv2 includes only one glyphs directory
        return [glyphsdir_path]


class Ufo3(Ufo):
    def __init__(self, ufopath, glyphsdir_list):
        super(Ufo3, self).__init__(ufopath, glyphsdir_list)
        self.ufopath = ufopath
        self.ufoversion = 3
        self.glyphsdir_list = glyphsdir_list
        self.mandatory_root_basefilepaths = ["metainfo.plist", "layercontents.plist"]
        self.mandatory_glyphsdir_basefilepaths = ["contents.plist"]
        self.all_root_plist_files_list = [
            "metainfo.plist",
            "fontinfo.plist",
            "groups.plist",
            "kerning.plist",
            "lib.plist",
            "layercontents.plist",
        ]
        self.all_glyphsdir_plist_files_list = ["contents.plist", "layerinfo.plist"]

    def get_glyphsdir_path_list(self):
        glyphsdir_path_list = []
        for glyphsdir in self.glyphsdir_list:
            glyphsdir_path = os.path.join(self.ufopath, glyphsdir[1])
            glyphsdir_path_list.append(glyphsdir_path)
        return glyphsdir_path_list



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import sys

from fontTools.ufoLib.glifLib import GlyphSet, glyphNameToFileName

from ufolint.data.tstobj import Result
from ufolint.stdoutput import StdStreamer


class GlifObj(object):
    """
    A simple object for use in ufoLib attribute assignments for *.glif file testing.
    """

    def __init__(self):
        pass


def run_all_glif_validations(ufoobj):
    glyphsdir_path_list = ufoobj.get_glyphsdir_path_list()
    ufoversion = ufoobj.get_ufo_version()
    ss = StdStreamer(ufoobj.ufopath)
    test_error_list = []
    for (
        glyphsdir
    ) in glyphsdir_path_list:  # for each directory that containts .glif files
        print(" ")
        sys.stdout.write(" - " + glyphsdir + "  ")
        sys.stdout.flush()
        res = Result(glyphsdir)
        try:
            gs = GlyphSet(
                glyphsdir, ufoFormatVersion=ufoversion, validateRead=True
            )  # create a ufoLib GlyphSet
            # do not report success for this, previous testing has passed this
        except Exception as e:
            res.test_failed = True
            res.test_long_stdstream_string = (
                " Failed to read glif file paths from "
                + glyphsdir
                + ". Error: "
                + str(e)
            )
            ss.stream_result(res)
            test_error_list.append(res)
            break  # break out loop as it was not possible to read the GlyphSet for this directory, gs not instantiated

        glif_count = 0  # reset glyphs directory .glif file counter
        for (
            glyphname
        ) in (
            gs.contents.keys()
        ):  # for each .glif file (read from glyph name in glyph set contents dict)
            res = Result(gs.contents[glyphname])
            try:
                go = GlifObj()
                gs.readGlyph(
                    glyphname, glyphObject=go
                )  # read the glif file and perform ufoLib validations, requires the glyphObject for validations
                res.test_failed = False
                ss.stream_result(res)
                glif_count += 1
            except Exception as e:
                res.test_failed = True
                filename = os.path.join(glyphsdir, glyphNameToFileName(glyphname, None))
                res.test_long_stdstream_string = '{} (glyph "{}"): Test failed with error: {}'.format(
                    filename, glyphname, e
                )
                ss.stream_result(res)
                test_error_list.append(res)
                glif_count += 1
        print("   " + str(glif_count) + " .glif tests completed")
    return test_error_list



#!/usr/bin/env python
# -*- coding: utf-8 -*-


class Result(object):
    """
    Data object that maintains test result success status and test result message (intended for std output to user)
    """

    def __init__(self, filepath):
        self.test_filepath = filepath
        self.test_failed = (
            None
        )  # test result indicator, True = test failed, False = test succeeded
        self.exit_failure = (
            None
        )  # when set to True, sys.exit(1) raised immediately on this error result
        self.test_long_stdstream_string = (
            ""
        )  # maintains std output string to be presented to user for each test result



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os


def dir_exists(dirpath):
    """Tests for existence of a directory on the string filepath"""
    if os.path.exists(dirpath) and os.path.isdir(
        dirpath
    ):  # test that exists and is a directory
        return True
    else:
        return False


def file_exists(filepath):
    """Tests for existence of a file on the string filepath"""
    if os.path.exists(filepath) and os.path.isfile(
        filepath
    ):  # test that exists and is a file
        return True
    else:
        return False



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# ====================================================
# Copyright 2018 Source Foundry Authors
# MIT License
# ====================================================

import os
import sys

from commandlines import Command

from ufolint.settings import HELP, VERSION, USAGE
from ufolint.controllers.runner import MainRunner


def main():
    c = Command()

    if c.does_not_validate_missing_args():
        sys.stderr.write(
            "[ufolint] ERROR: Please include one or more UFO directory arguments with your command."
            + os.linesep
        )
        sys.exit(1)

    if c.is_help_request():
        print(HELP)
        sys.exit(0)
    elif c.is_version_request():
        print(VERSION)
        sys.exit(0)
    elif c.is_usage_request():
        print(USAGE)
        sys.exit(0)

    for argument in sys.argv:
        if argument[-4:] == ".ufo":
            hh = MainRunner(argument)
            hh.run()

    sys.exit(
        0
    )  # if the script completes without status code 1 SystemExit being raised, then all tests passed



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import sys
from plistlib import load

from fontTools.ufoLib import UFOReader

from ufolint.data.tstobj import Result
from ufolint.data.ufo import Ufo2, Ufo3
from ufolint.stdoutput import StdStreamer
from ufolint.utilities import file_exists, dir_exists
from ufolint.validators.glifvalidators import run_all_glif_validations
from ufolint.validators.imagesvalidators import run_all_images_validations
from ufolint.validators.plistvalidators import (
    MetainfoPlistValidator,
    FontinfoPlistValidator,
    GroupsPlistValidator,
)
from ufolint.validators.plistvalidators import (
    KerningPlistValidator,
    LibPlistValidator,
    ContentsPlistValidator,
)
from ufolint.validators.plistvalidators import (
    LayercontentsPlistValidator,
    LayerinfoPlistValidator,
)


class MainRunner(object):
    def __init__(self, ufopath):
        self.ufopath = ufopath
        self.ufolib_reader = None
        self.ufoversion = None
        self.failures_list = (
            []
        )  # list of strings that include all failures across all tests for final report
        self.ufo_glyphs_dir_list = (
            []
        )  # list of glyphs directory(ies) available in the source (>1 permitted in UFOv3+)
        self.ufoobj = None

    def run(self):
        # Print UFO filepath header
        print(" ")
        print("ufolint:")
        print("~" * len(self.ufopath))
        print(self.ufopath)
        print("~" * len(self.ufopath))
        print(" ")

        # [START] EARLY FAIL TESTS ----------------------------------------------------------------
        #      UFO directory filepath
        #      .ufo directory extension
        #      import with ufoLib
        #      version check
        #      ufo obj define
        #        v3 only: presence of layercontents.plist to define the glyphs directories in source
        #        v2 only: no layercontents.plist, define as single glyphs directory
        ss = StdStreamer(self.ufopath)
        ss.stream_testname("UFO directory")
        self._check_ufo_dir_path_exists()  # tests user defined UFO directory path
        self._check_ufo_dir_extension()  # tests for .ufo extension on directory
        self._check_metainfo_plist_exists()  # confirm presence of metainfo.plist to define UFO version
        self._validate_read_data_types_metainfo_plist()  # validate the version data type as integer (workaround for bug in ufoLib)
        self._check_ufo_import_and_define_ufo_version()  # confirm ufoLib can import directory. defines UFOReader object as class property
        if self.ufoversion == 3:
            self._check_layercontents_plist_exists()  # tests for presence of a layercontents.plist in root of UFO
            self._validate_read_load_glyphsdirs_layercontents_plist()  # validate layercontents.plist xml and load glyphs dirs
        elif self.ufoversion == 2:
            self.ufo_glyphs_dir_list = [
                ["public.default", "glyphs"]
            ]  # define as single glyphs directory for UFOv2
        else:  # pragma nocoverage fail if unsupported UFO version (ufolint fail in case behind released UFO version)
            sys.stderr.write(
                os.linesep
                + "[ufolint] UFO v"
                + self.ufoversion
                + " is not supported in ufolint"
                + os.linesep
            )
            sys.exit(1)
        print(" ")
        print("   Found UFO v" + str(self.ufoversion))
        print("   Detected glyphs directories: ")
        for glyphs_dir in self.ufo_glyphs_dir_list:
            # TODO: add glyphs directory validation here : confirm naming = 'glyphs.*'
            sys.stdout.write(
                "     -- " + glyphs_dir[1] + " "
            )  # display the name of the specified glyphs dirs
            res = Result(glyphs_dir[1])
            if dir_exists(
                os.path.join(self.ufopath, glyphs_dir[1])
            ):  # test for presence of specified glyphs dir
                res.test_failed = False
                ss.stream_result(res)
            else:
                res.test_failed = True
                res.exit_failure = True
                res.test_long_stdstream_string = (
                    "Unable to find the UFO directory '"
                    + glyphs_dir[1]
                    + "' defined in layercontents.plist"
                )
                ss.stream_result(res)
            print(" ")

        # create Ufo objects for subsequent tests - all Ufo object dependent tests must take place below this level
        if self.ufoversion == 2:
            self.ufoobj = Ufo2(self.ufopath, self.ufo_glyphs_dir_list)
        elif self.ufoversion == 3:
            self.ufoobj = Ufo3(self.ufopath, self.ufo_glyphs_dir_list)

        # [START] Mandatory file path tests
        ss.stream_testname("UFO v" + str(self.ufoversion) + " mandatory files")

        mandatory_file_list = self.ufoobj.get_mandatory_filepaths_list()
        for mandatory_file in mandatory_file_list:
            res = Result(mandatory_file)
            if file_exists(mandatory_file):
                res.test_failed = False
                ss.stream_result(res)
            else:
                res.test_failed = True
                res.exit_failure = True
                res.test_long_stdstream_string = (
                    mandatory_file + " was not found in " + self.ufopath
                )
                ss.stream_result(res)
        print(" ")
        # [END] Mandatory file path tests
        # [END] EARLY FAIL TESTS ----------------------------------------------------------------

        # [START] XML VALIDATION TESTS  -----------------------------------------------------------
        ss.stream_testname("XML formatting")
        meta_val = MetainfoPlistValidator(
            self.ufopath, self.ufoversion, self.ufo_glyphs_dir_list
        )
        fontinfo_val = FontinfoPlistValidator(
            self.ufopath, self.ufoversion, self.ufo_glyphs_dir_list
        )
        groups_val = GroupsPlistValidator(
            self.ufopath, self.ufoversion, self.ufo_glyphs_dir_list
        )
        kerning_val = KerningPlistValidator(
            self.ufopath, self.ufoversion, self.ufo_glyphs_dir_list
        )
        lib_val = LibPlistValidator(
            self.ufopath, self.ufoversion, self.ufo_glyphs_dir_list
        )
        contents_val = ContentsPlistValidator(
            self.ufopath, self.ufoversion, self.ufo_glyphs_dir_list
        )
        layercont_val = LayercontentsPlistValidator(
            self.ufopath, self.ufoversion, self.ufo_glyphs_dir_list
        )
        layerinfo_val = LayerinfoPlistValidator(
            self.ufopath, self.ufoversion, self.ufo_glyphs_dir_list
        )

        # excute validations, returns list of failure Result() objects
        mv_xml_fail_list = meta_val.run_xml_validation()
        fi_xml_fail_list = fontinfo_val.run_xml_validation()
        g_xml_fail_list = groups_val.run_xml_validation()
        k_xml_fail_list = kerning_val.run_xml_validation()
        l_xml_fail_list = lib_val.run_xml_validation()
        c_xml_fail_list = contents_val.run_xml_validation()
        lc_xml_fail_list = layercont_val.run_xml_validation()
        li_xml_fail_list = layerinfo_val.run_xml_validation()

        # xml validations return lists of all failures, append these to the class failures_list Python list
        for thelist in (
            mv_xml_fail_list,
            fi_xml_fail_list,
            g_xml_fail_list,
            k_xml_fail_list,
            l_xml_fail_list,
            c_xml_fail_list,
            lc_xml_fail_list,
            li_xml_fail_list,
        ):
            for failed_test_result in thelist:
                self.failures_list.append(failed_test_result)
        print(" ")
        # [END] XML VALIDATION TESTS  --------------------------------------------------------------

        # [START] plist FILE VALIDATION TESTS (includes numerous ufoLib library validations on plist file reads)
        ss.stream_testname("*.plist spec")
        mv_ufolib_import_fail_list = meta_val.run_ufolib_import_validation()
        fi_ufolib_import_fail_list = fontinfo_val.run_ufolib_import_validation()
        g_ufolib_import_fail_list = groups_val.run_ufolib_import_validation()
        k_ufolib_import_fail_list = kerning_val.run_ufolib_import_validation()
        l_ufolib_import_fail_list = lib_val.run_ufolib_import_validation()
        c_ufolib_import_fail_list = contents_val.run_ufolib_import_validation()
        lc_ufolib_import_fail_list = layercont_val.run_ufolib_import_validation()
        li_ufolib_import_fail_list = layerinfo_val.run_ufolib_import_validation()

        for thelist in (
            mv_ufolib_import_fail_list,
            fi_ufolib_import_fail_list,
            g_ufolib_import_fail_list,
            k_ufolib_import_fail_list,
            l_ufolib_import_fail_list,
            c_ufolib_import_fail_list,
            lc_ufolib_import_fail_list,
            li_ufolib_import_fail_list,
        ):
            for failed_test_result in thelist:
                self.failures_list.append(failed_test_result)

        # [END] plist FILE VALIDATION TESTS

        # [START] features.fea TESTS
        print(" ")
        ss.stream_testname("features.fea")
        ff_path = os.path.join(self.ufopath, "features.fea")
        res = Result(ff_path)
        if file_exists(ff_path):
            res.test_failed = False
            ss.stream_result(res)
        else:
            sys.stdout.write("not present")  # not a mandatory file, not a failure
        # [END] features.fea TESTS

        # [START] DATA DIRECTORY TESTS - no fails in these tests, reporting only
        if self.ufoversion == 3:  # UFOv3+ only
            print(" ")
            ss.stream_testname("data")
            data_dir_path = os.path.join(self.ufopath, "data")

            if dir_exists(data_dir_path):
                ufo_reader = UFOReader(self.ufopath, validate=True)
                raw_data_list = ufo_reader.getDataDirectoryListing()
                data_list = []
                for item in raw_data_list:
                    if (
                        not item[0] == "."
                    ):  # eliminate dotfiles picked up by the ufoLib method (e.g. .DS_Store on OSX)
                        data_list.append(item)
                if len(data_list) == 0:
                    sys.stdout.write("empty")
                else:
                    sys.stdout.write(str(len(data_list)) + " data files")
            else:
                sys.stdout.write(
                    "not present"
                )  # not a mandatory directory, not a failure
        # [END] DATA DIRECTORY TESTS

        # [START] IMAGES DIRECTORY TESTS
        if self.ufoversion == 3:  # UFO v3+ only
            print(" ")
            ss.stream_testname("images")
            images_dir_path = os.path.join(self.ufopath, "images")

            if dir_exists(images_dir_path):
                images_dir_failures = run_all_images_validations(self.ufoobj)
                for images_failure_result in images_dir_failures:
                    self.failures_list.append(images_failure_result)
            else:
                sys.stdout.write(
                    "not present"
                )  # not a mandatory directory, not a failure
        # [END] IMAGES DIRECTORY TESTS

        # [START] *.glif VALIDATION TESTS
        print(" ")
        ss.stream_testname("*.glif spec")
        glif_validation_failures = run_all_glif_validations(self.ufoobj)
        for glif_failure_result in glif_validation_failures:
            self.failures_list.append(glif_failure_result)
        # [END] *.glif VALIDATION TESTS

        # TESTS COMPLETED --------------------------------------------------------------------------
        #   stream all failure results as a newline delimited list to user and exit with status code 1
        #   if failures are present, status code 0 if failures are not present
        ss = StdStreamer(self.ufopath)
        ss.stream_final_failures(self.failures_list)

    # =====================================
    #
    #  TESTS
    #
    # =====================================

    def _check_layercontents_plist_exists(self):
        """
        UFO 3+ test for layercontents.plist file in the top level of UFO directory
        :return: None - method leads to early exit with status code 1 if file not found
        """
        ss = StdStreamer(self.ufopath)
        lcp_test_filepath = os.path.join(self.ufopath, "layercontents.plist")
        res = Result(lcp_test_filepath)

        if file_exists(lcp_test_filepath):
            res.test_failed = False
            ss.stream_result(res)
        else:
            res.test_failed = True
            res.exit_failure = (
                True
            )  # early exit if cannot find this file to define glyphs directories in UFO source
            res.test_long_stdstream_string = (
                "layercontents.plist was not found in " + self.ufopath
            )
            ss.stream_result(res)

    def _check_metainfo_plist_exists(self):
        """
        Test for presence of metainfo.plist file in the UFO directory. Mandatory file that defines UFO version.
        :return: None = method leads to early exit with status code 1 if file not found
        """
        ss = StdStreamer(self.ufopath)
        meta_test_filepath = os.path.join(self.ufopath, "metainfo.plist")
        res = Result(meta_test_filepath)

        if file_exists(meta_test_filepath):
            res.test_failed = False
            ss.stream_result(res)
        else:
            res.test_failed = True
            res.exit_failure = (
                True
            )  # early exit if cannot find this file to define glyphs directories in UFO source
            res.test_long_stdstream_string = (
                "metainfo.plist was not found in " + self.ufopath
            )
            ss.stream_result(res)

    def _check_ufo_import_and_define_ufo_version(self):
        """
        Tests UFO directory import with ufoLib UFOReader object and defines class property (ufo) with the
        ufoLib UFOReader object.  This object is used for additional tests in this module.  Failures added to the
        class property failures_list for final report
        :return: None
        """
        ss = StdStreamer(self.ufopath)
        res = Result(self.ufopath)
        try:
            ufolib_reader = UFOReader(self.ufopath, validate=True)
            self.ufoversion = ufolib_reader.formatVersion
            self.ufolib_reader = ufolib_reader
            res.test_failed = False
            ss.stream_result(res)
        except Exception as e:
            res.test_failed = True
            res.exit_failure = True
            res.test_long_stdstream_string = (
                "ufoLib raised an exception with import of "
                + self.ufopath
                + os.linesep
                + str(e)
            )
            self.failures_list.append(res)
            ss.stream_result(res)

    def _check_ufo_dir_extension(self):
        """
        Tests for .ufo extension on the user defined (command line) directory path. Results
        streamed through std output stream. Failures added to the
        class property failures_list for final report
        :return: None
        """
        ss = StdStreamer(self.ufopath)
        res = Result(self.ufopath)
        if self.ufopath[-4:] == ".ufo":
            res.test_failed = False
            ss.stream_result(res)
        else:
            res.test_failed = True
            res.exit_failure = True
            res.test_long_stdstream_string = (
                self.ufopath + " directory does not have a .ufo extension"
            )
            self.failures_list.append(res)
            ss.stream_result(res)

    def _check_ufo_dir_path_exists(self):
        """
        Tests existence of a directory on the user defined (command line) directory path
        Results streamed through std output stream. Failures added to the
        class property failures_list for final report
        :return: None
        """
        ss = StdStreamer(self.ufopath)
        if dir_exists(self.ufopath):
            res = Result(self.ufopath)
            res.test_failed = False
            ss.stream_result(res)

        else:
            res = Result(self.ufopath)
            res.test_failed = True
            res.exit_failure = True
            res.test_long_stdstream_string = (
                self.ufopath + " does not appear to be a valid UFO directory"
            )
            self.failures_list.append(res.test_long_stdstream_string)
            ss.stream_result(res)  # raises sys.exit(1) on this failure

    def _validate_read_data_types_metainfo_plist(self):
        metainfo_plist_path = os.path.join(self.ufopath, "metainfo.plist")
        res = Result(metainfo_plist_path)
        ss = StdStreamer(metainfo_plist_path)
        try:
            with open(metainfo_plist_path, "rb") as f:
                meta_dict = load(f)
                if "formatVersion" in meta_dict.keys():
                    if isinstance(meta_dict["formatVersion"], int):
                        res.test_failed = False
                        ss.stream_result(res)
                    else:
                        res.test_failed = True
                        res.exit_failure = True  # early exit if fails
                        res.test_long_stdstream_string = (
                            metainfo_plist_path
                            + " 'formatVersion' value must be specified as"
                            " an integer"
                        )
                        ss.stream_result(res)
                else:
                    res.test_failed = True
                    res.exit_failure = True  # early exit if fails
                    res.test_long_stdstream_string = (
                        "Failed to read the 'formatVersion' value in "
                        + metainfo_plist_path
                    )
                    ss.stream_result(res)
        except Exception as e:
            res.test_failed = True
            res.exit_failure = True  # early exit if fails
            res.test_long_stdstream_string = (
                "Failed to read the 'formatVersion' value in "
                "the file " + metainfo_plist_path + ". Error: " + str(e)
            )
            ss.stream_result(res)

    def _validate_read_load_glyphsdirs_layercontents_plist(self):
        layercontents_plist_path = os.path.join(self.ufopath, "layercontents.plist")
        res = Result(layercontents_plist_path)
        ss = StdStreamer(layercontents_plist_path)
        try:
            # loads as [ ['layername1', 'glyphsdir1'], ['layername2', 'glyphsdir2'] ]
            with open(layercontents_plist_path, "rb") as f:
                self.ufo_glyphs_dir_list = load(f)
                res.test_failed = False
                ss.stream_result(res)
        except Exception as e:
            res.test_failed = True
            res.exit_failure = True
            res.test_long_stdstream_string = (
                "Failed to read " + layercontents_plist_path + ". Error: " + str(e)
            )
            ss.stream_result(res)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import sys
import os
from collections import OrderedDict


class StdStreamer(object):
    def __init__(self, ufo_path):
        self.ufopath = ufo_path
        self.short_success_string = "."
        self.short_fail_string = "F"
        self.fail_long_indicator = "[FAIL]"

    def _stream_short(self, output_string):
        sys.stdout.write(output_string)
        sys.stdout.flush()

    def stream_testname(self, testname_string):
        sys.stdout.write("[" + testname_string + "] ")
        sys.stdout.flush()

    def stream_results_list(self, result_list):
        for resultobj in result_list:
            self.stream_result(resultobj)

    def stream_result(self, resultobj):
        if resultobj.test_failed is True:
            self._stream_short(self.short_fail_string)
            if resultobj.exit_failure is True:
                fail_string = (
                    self.ufopath + " failed ufolint testing! Exit with status code 1"
                )
                print(os.linesep)
                print("=" * len(fail_string))
                print(self.ufopath + " failed ufolint testing! Exit with status code 1")
                print("=" * len(fail_string))
                print(" ")
                print("Test result that led to failure:")
                print(
                    self.fail_long_indicator
                    + " "
                    + resultobj.test_long_stdstream_string
                )
                print(" ")  # add newline to stdout before exiting
                sys.exit(1)
        elif resultobj.test_failed is False:
            self._stream_short(self.short_success_string)

    def stream_final_failures(self, result_list):
        if len(result_list) > 0:
            print(" ")
            fail_string = (
                self.ufopath + " failed ufolint testing! Exit with status code 1"
            )
            print(" ")
            print("=" * len(fail_string))
            print(fail_string)
            print("=" * len(fail_string))
            print(" ")

            # remove duplicates by making an OrderedDict
            result_list_deduped = OrderedDict((x, True) for x in result_list).keys()

            for resultobj in result_list_deduped:
                print(
                    self.fail_long_indicator
                    + " "
                    + resultobj.test_long_stdstream_string
                )
            sys.exit(1)
        else:
            success_string = self.ufopath + " - All ufolint tests passed!"
            print(" ")
            print(" ")
            print("=" * len(success_string))
            print(success_string)
            print("=" * len(success_string))



import io
import os
import re
import sys
from setuptools import setup, find_packages

REQUIRES_PYTHON = ">=3.6.0"

# Optional packages
EXTRAS_REQUIRES = {
    # for developer installs
    "dev": ["wheel", "setuptools", "twine", "coverage", "pytest", "tox", "flake8", "pytype"]
}


def version_read():
    settings_file = open(os.path.join(os.path.dirname(__file__), 'lib', 'ufolint', 'settings.py')).read()
    major_regex = r"""major_version\s*?=\s*?["']{1}(\d+)["']{1}"""
    minor_regex = r"""minor_version\s*?=\s*?["']{1}(\d+)["']{1}"""
    patch_regex = r"""patch_version\s*?=\s*?["']{1}(\d+)["']{1}"""
    major_match = re.search(major_regex, settings_file)
    minor_match = re.search(minor_regex, settings_file)
    patch_match = re.search(patch_regex, settings_file)
    major_version = major_match.group(1)
    minor_version = minor_match.group(1)
    patch_version = patch_match.group(1)
    if len(major_version) == 0:
        major_version = 0
    if len(minor_version) == 0:
        minor_version = 0
    if len(patch_version) == 0:
        patch_version = 0
    return major_version + "." + minor_version + "." + patch_version


# Use repository Markdown README.md for PyPI long description
try:
    with io.open("README.md", encoding="utf-8") as f:
        readme = f.read()
except IOError as readme_e:
    sys.stderr.write(
        "[ERROR] setup.py: Failed to read the README.md file for the long description definition: {}".format(
            str(readme_e)
        )
    )
    raise readme_e

setup(
    name='ufolint',
    version=version_read(),
    description='UFO source file linter',
    long_description=readme,
    long_description_content_type="text/markdown",
    url='https://github.com/source-foundry/ufolint',
    license='MIT license',
    author='Christopher Simpkins',
    author_email='chris@sourcefoundry.org',
    platforms=['any'],
    packages=find_packages("lib"),
    package_dir={'': 'lib'},
    install_requires=['commandlines', 'fontTools[ufo]>=3.31.0'],
    entry_points={
        'console_scripts': [
            'ufolint = ufolint.app:main'
        ],
    },
    keywords='',
    include_package_data=True,
    extras_require=EXTRAS_REQUIRES,
    python_requires=REQUIRES_PYTHON,
    classifiers=[
        'Development Status :: 4 - Beta',
        'Natural Language :: English',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Topic :: Software Development :: Libraries :: Python Modules',
        "Topic :: Text Processing :: Fonts",
    ],
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os

from fontTools.ufoLib.validators import pngValidator

from ufolint.utilities import file_exists, dir_exists
from ufolint.data.tstobj import Result
from ufolint.stdoutput import StdStreamer


def run_all_images_validations(ufoobj):
    """
    Tests images directory without testing to confirm that directory is present.  Directory existence testing performed
    in calling code.  Uses ufoLib.validators.pngValidator public method to validate files identified in 'images' dir
    :param ufoobj: ufolint.data.ufo.Ufo object
    :return: (list) list of failed test results as ufolint.tstobj.Result objects
    """
    test_error_list = []
    ss = StdStreamer(ufoobj.ufopath)
    images_dir_path = os.path.join(ufoobj.ufopath, "images")

    if dir_exists(images_dir_path) is False:
        return (
            []
        )  # if the directory path does not exist, return an empty test_error_list to calling code

    for testimage_rel_path in os.listdir(images_dir_path):
        testimage_path = os.path.join(images_dir_path, testimage_rel_path)
        if file_exists(testimage_path):
            if (
                testimage_rel_path[0] == "."
            ):  # ignore files that are dotfiles in directory (e.g. .DS_Store on OS X)
                pass
            else:
                passed_ufolib_tests, error = pngValidator(
                    path=testimage_path
                )  # call ufoLib PNG validator directly
                res = Result(testimage_path)

                if passed_ufolib_tests is True:
                    res.test_failed = False
                    ss.stream_result(res)
                else:
                    res.test_failed = True
                    res.test_long_stdstream_string = (
                        testimage_path + " failed with error: " + error
                    )
                    test_error_list.append(
                        res
                    )  # add to error list returned to calling code
                    ss.stream_result(res)
    return test_error_list  # return list of identified errors to the calling code
